@@ -212,7 +212,8 @@ combine.dsmdata.f <- function(blocks = NULL, dists = NULL) {
 
 
 
-get.direction.unit.f <- function(data = NULL, is.blocks = T, geometry = "euc") {
+get.direction.unit.f <- function(data = NULL, is.blocks = TRUE,
+                                 geometry = "euc") {
 
   # NOTE: old param: data=data
 
@@ -222,8 +223,7 @@ get.direction.unit.f <- function(data = NULL, is.blocks = T, geometry = "euc") {
   if (is.blocks) {
     data$Unit <- data$Block.Label
     name.label <- "Block.Label"
-  }
-  else {
+  } else {
     data$Unit <- data$Transect.Label
     name.label <- "Transect.Label"
   }
@@ -566,7 +566,8 @@ get.triangle.sides.f <- function(seg.len = NULL, angle = NULL) {
 
 
 
-generate.obs.location.f <- function(seg = NULL, dists = NULL, geometry = "euc", do.plot = F) {
+generate.obs.location.f <- function(seg = NULL, dists = NULL, geometry = "euc",
+                                    do.plot = FALSE) {
   #
   # NOTE the parameter seg used to be "seg=segments", which caused CRAN compatibility issues
   # same for dists=distance.data
