@@ -121,7 +121,13 @@ bru_compute_linearisation.component <- function(cmp,
       } else {
         values <- (pred_eps - pred0)
       }
-      nonzero <- (values != 0.0) # Detect exact (non)zeros
+      nonzero <- is.finite(values)
+      if (any(!nonzero)) {
+        warning("Non-finite (-Inf/Inf/NaN) entries detected in predictor derivatives; treated as 0.0",
+          immediate. = TRUE
+        )
+      }
+      nonzero[nonzero] <- (values[nonzero] != 0.0) # Detect exact (non)zeros
       if (assume_rowwise) {
         triplets$i <- c(triplets$i, row_subset[nonzero])
       } else {
