@@ -1,5 +1,6 @@
 \donttest{
-  if (require(ggplot2, quietly = TRUE)) {
+  if (require(ggplot2, quietly = TRUE) &&
+      require(ggpolypath, quietly = TRUE)) {
     # Load Gorilla data
 
     data("gorillas", package = "inlabru")
